"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const fs_1 = require("fs");
const command_1 = require("@oclif/command");
const listr_1 = tslib_1.__importDefault(require("listr"));
const base_command_1 = tslib_1.__importDefault(require("../base-command"));
const configure_aws_1 = require("../configure-aws");
const trick_registry_1 = require("../tricks/trick-registry");
const shutdown_ec2_instances_trick_1 = require("../tricks/shutdown-ec2-instances.trick");
const stop_fargate_ecs_services_trick_1 = require("../tricks/stop-fargate-ecs-services.trick");
const stop_rds_database_instances_trick_1 = require("../tricks/stop-rds-database-instances.trick");
const decrease_dynamodb_provisioned_rcu_wcu_trick_1 = require("../tricks/decrease-dynamodb-provisioned-rcu-wcu.trick");
const remove_nat_gateways_trick_1 = require("../tricks/remove-nat-gateways.trick");
const snapshot_remove_elasticache_redis_trick_1 = require("../tricks/snapshot-remove-elasticache-redis.trick");
const decrease_kinesis_streams_shards_trick_1 = require("../tricks/decrease-kinesis-streams-shards.trick");
const stop_rds_database_clusters_trick_1 = require("../tricks/stop-rds-database-clusters.trick");
const scaledown_auto_scaling_groups_trick_1 = require("../tricks/scaledown-auto-scaling-groups.trick");
const suspend_auto_scaling_groups_trick_1 = require("../tricks/suspend-auto-scaling-groups.trick");
class Conserve extends base_command_1.default {
    async run() {
        const { flags } = this.parse(Conserve);
        const awsConfig = await configure_aws_1.configureAWS(flags.profile, flags.region);
        await this.validateStateFilePath(flags);
        this.printBanner(awsConfig, flags);
        const tricks = this.getEnabledTricks(flags);
        const rootState = {};
        const rootTaskList = [];
        const options = {
            dryRun: flags['dry-run'],
        };
        for (const trick of tricks) {
            rootTaskList.push({
                title: `${trick.getConserveTitle()}`,
                task: () => this.createTrickListr(rootState, trick, options),
            });
        }
        await new listr_1.default(rootTaskList, {
            renderer: process.env.NODE_ENV === 'test' ? 'silent' : 'default',
            concurrent: true,
            exitOnError: false,
            collapse: true,
        })
            .run(rootState)
            .finally(() => {
            if (!flags['no-state-file']) {
                fs_1.writeFileSync(flags['state-file'], JSON.stringify(rootState, null, 2), 'utf-8');
                this.log(`\n  ${chalk_1.default.green('❯')} Wrote state file to ${chalk_1.default.green(flags['state-file'])}`);
            }
        })
            .then(() => {
            if (flags['dry-run']) {
                this.log(`\n${chalk_1.default.yellow(' ↓ Skipped conserve due to dry-run.')}`);
            }
            else {
                this.log(`\n ${chalk_1.default.green('✔')} Successfully conserved.`);
            }
        })
            .catch(error => {
            if (error.errors.length < rootTaskList.length) {
                this.log(`\n${chalk_1.default.yellow('✔')} Partially conserved, with ${chalk_1.default.red(`${error.errors.length} error(s)`)}.`);
            }
            else {
                this.log(`\n${chalk_1.default.red('✖')} All ${chalk_1.default.red(`${rootTaskList.length} tricks failed`)} with errors.`);
            }
        });
    }
    createTrickListr(rootState, trick, options) {
        rootState[trick.getMachineName()] = [];
        return new listr_1.default([
            {
                title: `Fetch current state`,
                task: (ctx, task) => trick.getCurrentState(task, rootState[trick.getMachineName()], options),
            },
            {
                title: `Conserve resources`,
                task: (ctx, task) => trick.conserve(task, rootState[trick.getMachineName()], options),
            },
        ], {
            concurrent: false,
            exitOnError: true,
            collapse: false,
        });
    }
    getEnabledTricks(flags) {
        const tricksRegistry = trick_registry_1.TrickRegistry.initialize();
        const tricks = tricksRegistry.all();
        let enabledTricks = flags['no-default-tricks']
            ? []
            : [].concat(...Conserve.tricksEnabledByDefault);
        if (flags['use-trick'] && flags['use-trick'].length > 0) {
            enabledTricks.push(...flags['use-trick']);
        }
        if (flags['ignore-trick'] && flags['ignore-trick'].length > 0) {
            enabledTricks = enabledTricks.filter(trickName => !flags['ignore-trick'].includes(trickName));
        }
        return enabledTricks.map(trickName => {
            const trick = tricks.find(trick => trick.getMachineName() === trickName);
            if (!trick) {
                this.log(`Could not find a trick named ${chalk_1.default.yellow(trickName)}`);
                this.log(`Run ${chalk_1.default.yellow('aws-cost-saver conserve --help')} to get a list of available tricks.\n`);
                throw new Error('TrickNotFound');
            }
            return trick;
        });
    }
}
exports.default = Conserve;
Conserve.tricksEnabledByDefault = [
    shutdown_ec2_instances_trick_1.ShutdownEC2InstancesTrick.machineName,
    stop_fargate_ecs_services_trick_1.StopFargateEcsServicesTrick.machineName,
    stop_rds_database_instances_trick_1.StopRdsDatabaseInstancesTrick.machineName,
    decrease_dynamodb_provisioned_rcu_wcu_trick_1.DecreaseDynamoDBProvisionedRcuWcuTrick.machineName,
    decrease_kinesis_streams_shards_trick_1.DecreaseKinesisStreamsShardsTrick.machineName,
    stop_rds_database_clusters_trick_1.StopRdsDatabaseClustersTrick.machineName,
    suspend_auto_scaling_groups_trick_1.SuspendAutoScalingGroupsTrick.machineName,
];
Conserve.tricksDisabledByDefault = [
    // Removing NAT gateways can confuse IaC like terraform
    remove_nat_gateways_trick_1.RemoveNatGatewaysTrick.machineName,
    // This is an experimental trick, plus removing and recreating ElastiCache clusters takes a long time
    snapshot_remove_elasticache_redis_trick_1.SnapshotRemoveElasticacheRedisTrick.machineName,
    // Scaling-down an ASG will cause all instances to be terminated and lose their temporary volumes
    scaledown_auto_scaling_groups_trick_1.ScaledownAutoScalingGroupsTrick.machineName,
];
Conserve.description = [
    `This command uses various tricks to conserve as much money as possible.`,
    `To restore, this command will create a \`aws-cost-saver.json\` file to be use by "restore".`,
    `\nThese tricks are ${chalk_1.default.bold('enabled')} by default: ${chalk_1.default.green(`\n\t- ${Conserve.tricksEnabledByDefault.join('\n\t- ')}`)}`,
    `\nThese tricks are ${chalk_1.default.bold('disabled')} by default: ${chalk_1.default.redBright(`\n\t- ${Conserve.tricksDisabledByDefault.join('\n\t- ')}`)}`,
].join('\n');
Conserve.examples = [
    `$ aws-cost-saver conserve`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('--dry-run')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('--no-state-file')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--use-trick ${chalk_1.default.bold(snapshot_remove_elasticache_redis_trick_1.SnapshotRemoveElasticacheRedisTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--ignore-trick ${chalk_1.default.bold(stop_rds_database_instances_trick_1.StopRdsDatabaseInstancesTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--no-default-tricks --use-trick ${chalk_1.default.bold(stop_fargate_ecs_services_trick_1.StopFargateEcsServicesTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--region ${chalk_1.default.bold(`eu-central-1`)} --profile ${chalk_1.default.bold(`my-aws-profile`)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--state-file ${chalk_1.default.bold(`new-path.json`)}`)}`,
];
Conserve.flags = {
    help: command_1.flags.help({ char: 'h' }),
    region: command_1.flags.string({ char: 'r', default: 'eu-central-1' }),
    profile: command_1.flags.string({ char: 'p', default: 'default' }),
    'dry-run': command_1.flags.boolean({
        char: 'd',
        description: 'Only print actions and write state-file of current resources.',
    }),
    'state-file': command_1.flags.string({
        char: 's',
        default: 'aws-cost-saver.json',
        description: 'Where to keep original state of stopped/decreased resources to restore later.',
    }),
    'no-state-file': command_1.flags.boolean({
        char: 'n',
        default: false,
        description: 'Ignore saving current state, useful when want to only conserve as much money as possible.',
    }),
    'use-trick': command_1.flags.string({
        char: 'u',
        multiple: true,
        description: 'Enables an individual trick. Useful for tricks that are disabled by default.',
    }),
    'ignore-trick': command_1.flags.string({
        char: 'i',
        multiple: true,
        description: 'Disables an individual trick. Useful when you do not like to use a specific trick.',
    }),
    'no-default-tricks': command_1.flags.boolean({
        default: false,
        description: 'Disables all default tricks. Useful alongside --use-trick when you only want a set of specific tricks to execute.',
    }),
};
Conserve.args = [];
//# sourceMappingURL=conserve.js.map