"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const fs_1 = require("fs");
const command_1 = require("@oclif/command");
const listr_1 = tslib_1.__importDefault(require("listr"));
const base_command_1 = tslib_1.__importDefault(require("../base-command"));
const trick_registry_1 = require("../tricks/trick-registry");
const configure_aws_1 = require("../configure-aws");
class Restore extends base_command_1.default {
    async run() {
        const { flags } = this.parse(Restore);
        const awsConfig = await configure_aws_1.configureAWS(flags.profile, flags.region);
        this.printBanner(awsConfig, flags);
        const tricksRegistry = trick_registry_1.TrickRegistry.initialize();
        const stateContent = fs_1.readFileSync(flags['state-file'], 'utf-8');
        const rootState = JSON.parse(stateContent.toString());
        const taskList = [];
        const options = {
            dryRun: flags['dry-run'],
        };
        for (const trick of tricksRegistry.all()) {
            taskList.push({
                title: `${trick.getRestoreTitle()}`,
                task: async (ctx, task) => {
                    if (!rootState[trick.getMachineName()]) {
                        return task.skip('Nothing was conserved previously.');
                    }
                    return trick
                        .restore(task, rootState[trick.getMachineName()], options)
                        .catch(task.report);
                },
                exitOnError: false,
                collapse: false,
            });
        }
        await new listr_1.default(taskList, {
            renderer: process.env.NODE_ENV === 'test' ? 'silent' : 'default',
            concurrent: true,
            exitOnError: false,
            collapse: true,
        })
            .run()
            .then(() => {
            if (flags['dry-run']) {
                this.log(`\n${chalk_1.default.yellow(' ↓ Skipped restore due to dry-run.')}`);
            }
            else {
                this.log(`\n ${chalk_1.default.green('✔')} Successfully restored state from ${chalk_1.default.green(flags['state-file'])}`);
            }
        })
            .catch(error => {
            this.log(`\n${chalk_1.default.yellow('✔')} Partially restored, with ${chalk_1.default.red(`${error.errors.length} errors`)}.`);
        });
    }
}
exports.default = Restore;
Restore.description = 'To restore AWS resources stopped by the conserve command.';
Restore.examples = [
    `$ aws-cost-saver restore`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow('--dry-run')}`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow(`--region ${chalk_1.default.bold(`eu-central-1`)} --profile ${chalk_1.default.bold(`my-aws-profile`)}`)}`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow(`--state-file ${chalk_1.default.bold(`another-path.json`)}`)}`,
];
Restore.flags = {
    help: command_1.flags.help({ char: 'h' }),
    region: command_1.flags.string({ char: 'r', default: 'eu-central-1' }),
    profile: command_1.flags.string({ char: 'p', default: 'default' }),
    'dry-run': command_1.flags.boolean({
        char: 'd',
        description: 'Only print actions but do not do them',
    }),
    'state-file': command_1.flags.string({
        char: 's',
        default: 'aws-cost-saver.json',
        description: 'Path to state-file which contains original state of AWS resources to restore to.',
    }),
};
//# sourceMappingURL=restore.js.map