"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecreaseKinesisStreamsShardsTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
class DecreaseKinesisStreamsShardsTrick {
    constructor() {
        this.ksClient = new aws_sdk_1.default.Kinesis();
    }
    getMachineName() {
        return DecreaseKinesisStreamsShardsTrick.machineName;
    }
    getConserveTitle() {
        return 'Decrease Kinesis Streams Shards';
    }
    getRestoreTitle() {
        return 'Restore Kinesis Streams Shards';
    }
    async getCurrentState(task, currentState, options) {
        const streamNames = await this.listKinesisStreamsNames(task);
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (!streamNames || streamNames.length === 0) {
            task.skip('No Kinesis Streams found');
            return subListr;
        }
        subListr.add(streamNames.map((streamName) => {
            const streamState = {
                name: streamName,
            };
            currentState.push(streamState);
            return {
                title: streamName,
                task: async (ctx, task) => this.getStreamState(task, streamState),
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 5,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const stream of currentState) {
                subListr.add({
                    title: `${chalk_1.default.blueBright(stream.name)}`,
                    task: (ctx, task) => this.conserveStreamShards(task, stream, options),
                });
            }
        }
        else {
            task.skip(`No Kinesis Stream found`);
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 5,
            exitOnError: false,
            collapse: false,
        });
        if (originalState && originalState.length > 0) {
            for (const table of originalState) {
                subListr.add({
                    title: chalk_1.default.blueBright(table.name),
                    task: (ctx, task) => this.restoreStreamShards(task, table, options),
                });
            }
        }
        else {
            task.skip(`No Kinesis Stream was previously conserved`);
        }
        return subListr;
    }
    async listKinesisStreamsNames(task) {
        const streamNames = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        streamNames.push(...(await this.ksClient.listStreams({ Limit: 100 }).promise())
            .StreamNames);
        return streamNames;
    }
    async getStreamState(task, streamState) {
        task.output = 'Fetching stream information...';
        const summary = await this.getStreamSummary(streamState.name);
        streamState.shards = summary.OpenShardCount;
        streamState.state = summary.StreamStatus;
    }
    async conserveStreamShards(task, streamState, options) {
        if (streamState.state !== 'ACTIVE') {
            task.skip(`State is not Active, it is ${streamState.state} instead.`);
            return;
        }
        if (streamState.shards < 2) {
            task.skip(`Shards are already at minimum of 1`);
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would decrease number of shards to 1`);
            return;
        }
        let targetShards = streamState.shards;
        let step = 0;
        do {
            targetShards = Math.ceil(targetShards / 2);
            step += 1;
            task.output = `Step #${step}: decreasing shards to ${targetShards}, final target: 1...`;
            await this.ksClient
                .updateShardCount({
                StreamName: streamState.name,
                TargetShardCount: targetShards,
                ScalingType: 'UNIFORM_SCALING',
            })
                .promise();
            await this.ksClient
                .waitFor('streamExists', {
                StreamName: streamState.name,
                $waiter: {
                    delay: 15,
                    maxAttempts: 100,
                },
            })
                .promise();
        } while (targetShards > 1);
        task.output = `Decreased number of shards to ${targetShards}`;
    }
    async restoreStreamShards(task, streamState, options) {
        if (streamState.state !== 'ACTIVE') {
            task.skip(`State was not Active, it was ${streamState.state} instead.`);
            return;
        }
        if (streamState.shards < 2) {
            task.skip(`Shards were already at minimum of 1`);
            return;
        }
        if (!streamState.shards) {
            task.skip(`Original number of shards is not found in state`);
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would increase number of shards to ${streamState.shards}`);
            return;
        }
        const streamSummary = await this.getStreamSummary(streamState.name);
        const currentShards = streamSummary.OpenShardCount;
        if (currentShards >= streamState.shards) {
            task.skip(`Stream shards are already configured to ${currentShards}. Previous number of shards: ${streamState.shards}`);
            return;
        }
        let targetShards = currentShards;
        let step = 0;
        do {
            targetShards = Math.min(Math.floor(targetShards * 2), streamState.shards);
            step += 1;
            task.output = `Step #${step}: increasing shards to ${targetShards}, final target: ${streamState.shards}...`;
            await this.ksClient
                .updateShardCount({
                StreamName: streamState.name,
                TargetShardCount: targetShards,
                ScalingType: 'UNIFORM_SCALING',
            })
                .promise();
            await this.ksClient
                .waitFor('streamExists', {
                StreamName: streamState.name,
                $waiter: {
                    delay: 15,
                    maxAttempts: 100,
                },
            })
                .promise();
        } while (targetShards < streamState.shards);
    }
    async getStreamSummary(streamName) {
        return (await this.ksClient
            .describeStreamSummary({
            StreamName: streamName,
        })
            .promise()).StreamDescriptionSummary;
    }
}
exports.DecreaseKinesisStreamsShardsTrick = DecreaseKinesisStreamsShardsTrick;
DecreaseKinesisStreamsShardsTrick.machineName = 'decrease-kinesis-streams-shards';
//# sourceMappingURL=decrease-kinesis-streams-shards.trick.js.map