"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoveNatGatewaysTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
class RemoveNatGatewaysTrick {
    constructor() {
        this.ec2Client = new aws_sdk_1.default.EC2();
    }
    getMachineName() {
        return RemoveNatGatewaysTrick.machineName;
    }
    getConserveTitle() {
        return 'Remove NAT Gateways';
    }
    getRestoreTitle() {
        return 'Recreate NAT Gateways';
    }
    async getCurrentState(task, currentState, options) {
        const natGateways = await this.listNatGateways(task);
        const subListr = new listr_1.default({
            concurrent: false,
            exitOnError: false,
            collapse: false,
        });
        subListr.add(natGateways.map((natGateway) => {
            return {
                title: natGateway.NatGatewayId || chalk_1.default.italic('<no-id>'),
                task: async (ctx, task) => {
                    if (!natGateway.NatGatewayId ||
                        !natGateway.VpcId ||
                        !natGateway.SubnetId ||
                        !natGateway.State ||
                        !natGateway.NatGatewayAddresses) {
                        throw new Error(`Unexpected values on Nat Gateway: ${JSON.stringify(natGateway)}`);
                    }
                    task.output = 'Finding routes...';
                    const routes = await this.findRoutes(natGateway);
                    currentState.push({
                        id: natGateway.NatGatewayId,
                        vpcId: natGateway.VpcId,
                        subnetId: natGateway.SubnetId,
                        state: natGateway.State,
                        routes,
                        allocationIds: natGateway.NatGatewayAddresses.map(a => a.AllocationId),
                        tags: natGateway.Tags,
                    });
                },
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 1,
            exitOnError: false,
            collapse: false,
        });
        for (const natGateway of currentState) {
            subListr.add({
                title: chalk_1.default.blueBright(`${natGateway.id} / ${RemoveNatGatewaysTrick.getNameTag(natGateway)}`),
                task: (ctx, task) => this.conserveNatGateway(task, natGateway, options),
            });
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 1,
            exitOnError: false,
            collapse: false,
        });
        for (const natGateway of originalState) {
            subListr.add({
                title: chalk_1.default.blueBright(natGateway.id),
                task: (ctx, task) => this.restoreNatGateway(task, natGateway, options),
            });
        }
        return subListr;
    }
    async conserveNatGateway(task, natGateway, options) {
        if (natGateway.state !== 'available') {
            task.skip(`Skipped, state is not "available", it is "${natGateway.state}" instead`);
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would remove NAT Gateway`);
            return;
        }
        task.output = 'Deleting NAT gateway...';
        await this.ec2Client
            .deleteNatGateway({
            NatGatewayId: natGateway.id,
        })
            .promise();
        task.output = 'NAT Gateway Deleted';
    }
    async restoreNatGateway(task, natGateway, options) {
        const allocationId = natGateway.allocationIds && natGateway.allocationIds[0];
        if (natGateway.state !== 'available') {
            task.skip(`Skipped, state was not "available", it was "${natGateway.state}" instead`);
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would create NAT Gateway on subnet = ${natGateway.subnetId} and allocate EIP = ${allocationId}`);
            return;
        }
        task.output = 'Creating NAT gateway...';
        const result = await this.ec2Client
            .createNatGateway({
            AllocationId: allocationId,
            SubnetId: natGateway.subnetId,
            TagSpecifications: [
                {
                    ResourceType: 'natgateway',
                    Tags: natGateway.tags,
                },
            ],
        })
            .promise();
        const newNatGatewayId = result.NatGateway
            .NatGatewayId;
        task.output = 'Waiting for NAT Gateway to be available...';
        await this.ec2Client
            .waitFor('natGatewayAvailable', {
            NatGatewayIds: [newNatGatewayId],
        })
            .promise();
        for (const route of natGateway.routes) {
            task.output = `Adding NAT gateway to original route tables (${route.destinationCidr ||
                route.destinationIpv6Cidr ||
                route.destinationPrefixListId})...`;
            await this.ec2Client
                .replaceRoute({
                RouteTableId: route.routeTableId,
                NatGatewayId: newNatGatewayId,
                DestinationCidrBlock: route.destinationCidr,
                DestinationIpv6CidrBlock: route.destinationIpv6Cidr,
                DestinationPrefixListId: route.destinationPrefixListId,
            })
                .promise();
        }
        task.output = 'NAT Gateway Created';
    }
    async listNatGateways(task) {
        const natGateways = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        natGateways.push(...(await this.ec2Client.describeNatGateways({ MaxResults: 10 }).promise()).NatGateways);
        return natGateways;
    }
    static getNameTag(natGateway) {
        return natGateway.tags
            .filter(t => t.Key.toString().toLowerCase() === 'name')
            .map(t => t.Value)
            .join(' ');
    }
    async findRoutes(natGateway) {
        const routes = [];
        const routeTables = await this.getAllRouteTables();
        routeTables.forEach(rt => {
            if (rt.Routes && rt.Routes.length > 0) {
                for (const route of rt.Routes) {
                    if (route.NatGatewayId === natGateway.NatGatewayId) {
                        routes.push({
                            routeTableId: rt.RouteTableId,
                            destinationCidr: route.DestinationCidrBlock,
                            destinationIpv6Cidr: route.DestinationIpv6CidrBlock,
                            destinationPrefixListId: route.DestinationPrefixListId,
                        });
                    }
                }
            }
        });
        return routes;
    }
    async getAllRouteTables() {
        if (!this.routeTablesCache) {
            // TODO Handle pagination
            const result = await this.ec2Client.describeRouteTables({}).promise();
            this.routeTablesCache = result.RouteTables;
        }
        return this.routeTablesCache;
    }
}
exports.RemoveNatGatewaysTrick = RemoveNatGatewaysTrick;
RemoveNatGatewaysTrick.machineName = 'remove-nat-gateways';
//# sourceMappingURL=remove-nat-gateways.trick.js.map