"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShutdownEC2InstancesTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class ShutdownEC2InstancesTrick {
    constructor() {
        this.ec2Client = new aws_sdk_1.default.EC2();
    }
    getMachineName() {
        return ShutdownEC2InstancesTrick.machineName;
    }
    getConserveTitle() {
        return 'Shutdown EC2 Instances';
    }
    getRestoreTitle() {
        return 'Start EC2 Instances';
    }
    async getCurrentState(task, currentState, options) {
        var _a;
        const reservations = await this.listReservations(task);
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (!reservations || reservations.length === 0) {
            task.skip('No EC2 instances found');
            return subListr;
        }
        for (const reservation of reservations) {
            subListr.add(((_a = reservation.Instances) === null || _a === void 0 ? void 0 : _a.map((instance) => {
                return {
                    title: instance.InstanceId || chalk_1.default.italic('<no-instance-id>'),
                    task: async () => {
                        if (!instance.InstanceId ||
                            !instance.State ||
                            !instance.State.Name) {
                            throw new Error(`Unexpected EC2 instance: ${JSON.stringify(instance)}`);
                        }
                        const nameTag = instance.Tags
                            ? instance.Tags.filter(t => t.Key.toString().toLowerCase() === 'name')
                                .map(t => t.Value)
                                .join(' ')
                            : '<no-name>';
                        const instanceState = {
                            id: instance.InstanceId,
                            state: instance.State.Name,
                            name: nameTag,
                        };
                        currentState.push(instanceState);
                    },
                };
            })) || []);
        }
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const instance of currentState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${instance.id} / ${instance.name}`),
                    task: (ctx, task) => this.conserveInstance(task, instance, options),
                });
            }
        }
        else {
            task.skip(`No EC2 instances found`);
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (originalState && originalState.length > 0) {
            for (const instance of originalState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${instance.id} / ${instance.name}`),
                    task: (ctx, task) => this.restoreInstance(task, instance, options),
                });
            }
        }
        else {
            task.skip(`No EC2 instances were conserved`);
        }
        return subListr;
    }
    async listReservations(task) {
        const reservations = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        reservations.push(...((await this.ec2Client.describeInstances({ MaxResults: 1000 }).promise()).Reservations || []));
        return reservations;
    }
    async conserveInstance(task, instanceState, options) {
        if (instanceState.state !== 'running') {
            task.skip(`Not in a "running" state instead in "${instanceState.state}" state`);
            return;
        }
        if (options.dryRun) {
            task.skip('Skipped, would stop the instance');
            return;
        }
        // TODO Stop multiple instances at a time
        task.output = 'Stopping EC2 instance...';
        await this.ec2Client
            .stopInstances({
            InstanceIds: [instanceState.id],
        })
            .promise();
        task.output = 'Waiting for EC2 instance to stop...';
        await this.ec2Client
            .waitFor('instanceStopped', {
            InstanceIds: [instanceState.id],
            $waiter: {
                delay: 10,
                maxAttempts: 60,
            },
        })
            .promise();
        task.output = 'Stopped';
    }
    async restoreInstance(task, instanceState, options) {
        if (instanceState.state !== 'running') {
            task.skip(`Was not in a "running" state instead in "${instanceState.state}" state`);
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would start the instance`);
            return;
        }
        task.output = 'Starting EC2 instance...';
        await this.ec2Client
            .startInstances({
            InstanceIds: [instanceState.id],
        })
            .promise();
        task.output = 'Waiting for EC2 instance to start...';
        await this.ec2Client
            .waitFor('instanceRunning', {
            InstanceIds: [instanceState.id],
            $waiter: {
                delay: 15,
                maxAttempts: 100,
            },
        })
            .promise();
        task.output = `Started`;
    }
}
exports.ShutdownEC2InstancesTrick = ShutdownEC2InstancesTrick;
ShutdownEC2InstancesTrick.machineName = 'shutdown-ec2-instances';
//# sourceMappingURL=shutdown-ec2-instances.trick.js.map