"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnapshotRemoveElasticacheRedisTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
class SnapshotRemoveElasticacheRedisTrick {
    constructor() {
        this.elcClient = new aws_sdk_1.default.ElastiCache();
    }
    getMachineName() {
        return SnapshotRemoveElasticacheRedisTrick.machineName;
    }
    getConserveTitle() {
        return 'Snapshot and Remove ElastiCache Redis Clusters';
    }
    getRestoreTitle() {
        return 'Recreate ElastiCache Redis Clusters from Snapshot';
    }
    async getCurrentState(task, currentState, options) {
        const replicationGroups = await this.listReplicationGroups(task);
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (!replicationGroups || replicationGroups.length === 0) {
            task.skip('No ElastiCache Redis clusters found');
            return subListr;
        }
        subListr.add(replicationGroups.map((replicationGroup) => {
            return {
                title: replicationGroup.ReplicationGroupId || chalk_1.default.italic('<no-id>'),
                task: async (ctx, task) => {
                    if (replicationGroup.ReplicationGroupId === undefined) {
                        throw new Error(`Unexpected error: ReplicationGroupId is missing for ElastiCache replication group`);
                    }
                    const replicationGroupState = {
                        id: replicationGroup.ReplicationGroupId,
                    };
                    currentState.push(replicationGroupState);
                    return this.getReplicationGroupState(task, replicationGroupState, replicationGroup);
                },
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const replicationGroup of currentState) {
                subListr.add({
                    title: chalk_1.default.blueBright(replicationGroup.id),
                    task: (ctx, task) => this.conserveReplicationGroup(task, replicationGroup, options),
                });
            }
        }
        else {
            task.skip(`No ElastiCache Redis clusters found`);
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (originalState && originalState.length > 0) {
            for (const replicationGroup of originalState) {
                subListr.add({
                    title: chalk_1.default.blueBright(replicationGroup.id),
                    task: (ctx, task) => this.restoreReplicationGroup(task, replicationGroup, options),
                });
            }
        }
        else {
            task.skip(`No ElastiCache Redis clusters found`);
        }
        return subListr;
    }
    async conserveReplicationGroup(task, replicationGroupState, options) {
        if (replicationGroupState.status !== 'available') {
            task.skip(`Skipped, current state is not "available" it is "${replicationGroupState.status}" instead`);
            return;
        }
        if (options.dryRun) {
            task.skip('Skipped, would take a snapshot and delete');
            return;
        }
        task.output = 'Taking a snapshot and deleting replication group...';
        await this.elcClient
            .deleteReplicationGroup({
            ReplicationGroupId: replicationGroupState.id,
            FinalSnapshotIdentifier: replicationGroupState.snapshotName,
        })
            .promise();
        task.output =
            'Waiting for snapshot to be created, and replication group to be deleted...';
        await this.elcClient
            .waitFor('replicationGroupDeleted', {
            ReplicationGroupId: replicationGroupState.id,
            $waiter: {
                delay: 30,
                maxAttempts: 100,
            },
        })
            .promise();
        task.output = `Snapshot ${replicationGroupState.snapshotName} taken and replication group deleted successfully`;
    }
    async restoreReplicationGroup(task, replicationGroupState, options) {
        if (!replicationGroupState.snapshotName) {
            throw new Error(`Unexpected error: snapshotName is missing in state`);
        }
        if (!replicationGroupState.createParams) {
            throw new Error(`Unexpected error: createParams is missing in state`);
        }
        if (replicationGroupState.status !== 'available') {
            task.skip(`Skipped, previous state was not "available" it was "${replicationGroupState.status}" instead`);
            return;
        }
        if (await this.doesReplicationGroupExist(replicationGroupState)) {
            task.skip('ElastiCache redis cluster already exists');
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would re-create the cluster and remove the snapshot ${replicationGroupState.snapshotName}`);
            return;
        }
        task.output = `Creating replication group (snapshot: ${replicationGroupState.snapshotName})...`;
        await this.elcClient
            .createReplicationGroup(replicationGroupState.createParams)
            .promise();
        task.output = `Waiting for replication group to be available (snapshot: ${replicationGroupState.snapshotName})...`;
        await this.elcClient
            .waitFor('replicationGroupAvailable', {
            ReplicationGroupId: replicationGroupState.id,
            $waiter: {
                delay: 30,
                maxAttempts: 100,
            },
        })
            .promise();
        task.output = `Deleting snapshot ${replicationGroupState.snapshotName}...`;
        await this.elcClient
            .deleteSnapshot({
            SnapshotName: replicationGroupState.snapshotName,
        })
            .promise();
        task.output = 'Restored successfully';
    }
    async getReplicationGroupState(task, replicationGroupState, replicationGroup) {
        replicationGroupState.status = 'unknown';
        if (replicationGroup.ARN === undefined) {
            throw new Error(`Unexpected error: ARN is missing for ElastiCache replication group`);
        }
        if (replicationGroup.Status === undefined) {
            throw new Error(`Unexpected error: Status is missing for ElastiCache replication group`);
        }
        if (!replicationGroup.MemberClusters ||
            replicationGroup.MemberClusters.length === 0) {
            throw new Error(`Unexpected error: No member clusters for ElastiCache replication group`);
        }
        if (replicationGroup.AuthTokenEnabled) {
            throw new Error(`Cannot conserve an AuthToken protected Redis Cluster because on restore token will change`);
        }
        task.output = 'Fetching a sample cache cluster...';
        const sampleCacheCluster = await this.describeCacheCluster(replicationGroup.MemberClusters[0]);
        if (!sampleCacheCluster) {
            throw new Error(`Could not find sample cache cluster`);
        }
        task.output = 'Preparing re-create params...';
        const snapshotName = SnapshotRemoveElasticacheRedisTrick.generateSnapshotName(replicationGroup);
        const nodeGroups = this.generateNodeGroupConfigs(replicationGroup);
        const createParams = await this.buildCreateParams(snapshotName, replicationGroup, nodeGroups, sampleCacheCluster);
        replicationGroupState.snapshotName = snapshotName;
        replicationGroupState.status = replicationGroup.Status;
        replicationGroupState.createParams = createParams;
    }
    generateNodeGroupConfigs(replicationGroup) {
        var _a;
        return (((_a = replicationGroup.NodeGroups) === null || _a === void 0 ? void 0 : _a.map(n => {
            var _a;
            return ({
                NodeGroupId: n.NodeGroupId,
                ReplicaCount: (_a = n.NodeGroupMembers) === null || _a === void 0 ? void 0 : _a.length,
                Slots: n.Slots,
            });
        })) || []);
    }
    async buildCreateParams(snapshotName, replicationGroup, nodeGroups, sampleCacheCluster) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        return {
            // Required
            ReplicationGroupId: replicationGroup.ReplicationGroupId,
            ReplicationGroupDescription: replicationGroup.Description ||
                `Restored by aws-cost-saver from snapshot: ${snapshotName}`,
            SnapshotName: snapshotName,
            // Replication Group Configs
            AtRestEncryptionEnabled: replicationGroup.AtRestEncryptionEnabled,
            TransitEncryptionEnabled: replicationGroup.TransitEncryptionEnabled,
            NumCacheClusters: nodeGroups.length > 1
                ? undefined
                : ((_a = replicationGroup.MemberClusters) === null || _a === void 0 ? void 0 : _a.length) || 1,
            NumNodeGroups: nodeGroups.length > 1 ? nodeGroups.length : undefined,
            MultiAZEnabled: replicationGroup.MultiAZ !== undefined &&
                replicationGroup.MultiAZ === 'enabled',
            NodeGroupConfiguration: nodeGroups.length > 1 ? nodeGroups : undefined,
            GlobalReplicationGroupId: (_b = replicationGroup.GlobalReplicationGroupInfo) === null || _b === void 0 ? void 0 : _b.GlobalReplicationGroupId,
            KmsKeyId: replicationGroup.KmsKeyId,
            AutomaticFailoverEnabled: replicationGroup.AutomaticFailover !== undefined &&
                ['enabled', 'enabling'].includes(replicationGroup.AutomaticFailover),
            Port: sampleCacheCluster.CacheNodes && ((_d = (_c = sampleCacheCluster.CacheNodes[0]) === null || _c === void 0 ? void 0 : _c.Endpoint) === null || _d === void 0 ? void 0 : _d.Port),
            Tags: await this.getTags(sampleCacheCluster.ARN),
            // Cache Clusters Configs
            PreferredCacheClusterAZs: nodeGroups.length > 1
                ? undefined
                : sampleCacheCluster.PreferredAvailabilityZone
                    ? [sampleCacheCluster.PreferredAvailabilityZone]
                    : undefined,
            SecurityGroupIds: (_e = sampleCacheCluster.SecurityGroups) === null || _e === void 0 ? void 0 : _e.map(s => s.SecurityGroupId),
            CacheSecurityGroupNames: (_f = sampleCacheCluster.CacheSecurityGroups) === null || _f === void 0 ? void 0 : _f.map(c => c.CacheSecurityGroupName || ''),
            // Shared Cache Cluster Configs
            AutoMinorVersionUpgrade: sampleCacheCluster.AutoMinorVersionUpgrade,
            CacheNodeType: sampleCacheCluster.CacheNodeType,
            SnapshotRetentionLimit: sampleCacheCluster.SnapshotRetentionLimit,
            Engine: sampleCacheCluster.Engine,
            EngineVersion: sampleCacheCluster.EngineVersion,
            CacheParameterGroupName: (_g = sampleCacheCluster.CacheParameterGroup) === null || _g === void 0 ? void 0 : _g.CacheParameterGroupName,
            CacheSubnetGroupName: sampleCacheCluster.CacheSubnetGroupName,
            NotificationTopicArn: (_h = sampleCacheCluster.NotificationConfiguration) === null || _h === void 0 ? void 0 : _h.TopicArn,
            SnapshotWindow: sampleCacheCluster.SnapshotWindow,
            PreferredMaintenanceWindow: sampleCacheCluster.PreferredMaintenanceWindow,
        };
    }
    async doesReplicationGroupExist(replicationGroupState) {
        const result = await this.elcClient
            .describeReplicationGroups({
            ReplicationGroupId: replicationGroupState.id,
        })
            .promise();
        return Boolean(result.ReplicationGroups && result.ReplicationGroups.length > 0);
    }
    async listReplicationGroups(task) {
        const replicationGroups = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        replicationGroups.push(...(await this.elcClient
            .describeReplicationGroups({ MaxRecords: 100 })
            .promise()).ReplicationGroups);
        return replicationGroups;
    }
    async describeCacheCluster(cacheClusterId) {
        return (await this.elcClient
            .describeCacheClusters({
            CacheClusterId: cacheClusterId,
            ShowCacheNodeInfo: true,
        })
            .promise()).CacheClusters.pop();
    }
    async getTags(arn) {
        return (await this.elcClient
            .listTagsForResource({
            ResourceName: arn,
        })
            .promise()).TagList;
    }
    static generateSnapshotName(replicationGroup) {
        const now = new Date();
        return `${replicationGroup.ReplicationGroupId}-${now.getFullYear()}${now.getMonth()}${now.getDay()}${now.getHours()}${now.getMinutes()}`;
    }
}
exports.SnapshotRemoveElasticacheRedisTrick = SnapshotRemoveElasticacheRedisTrick;
SnapshotRemoveElasticacheRedisTrick.machineName = 'snapshot-remove-elasticache-redis';
//# sourceMappingURL=snapshot-remove-elasticache-redis.trick.js.map