"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StopFargateEcsServicesTrick = void 0;
const tslib_1 = require("tslib");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
class StopFargateEcsServicesTrick {
    constructor() {
        this.ecsClient = new aws_sdk_1.default.ECS();
        this.aasClient = new aws_sdk_1.default.ApplicationAutoScaling();
    }
    getMachineName() {
        return StopFargateEcsServicesTrick.machineName;
    }
    getConserveTitle() {
        return 'Stop Fargate ECS Services';
    }
    getRestoreTitle() {
        return 'Restore Fargate ECS Services';
    }
    async getCurrentState(task, currentState, options) {
        const clustersArn = await this.listClusters(task);
        const subListr = new listr_1.default({
            concurrent: 3,
            exitOnError: false,
            collapse: false,
        });
        if (!clustersArn || clustersArn.length === 0) {
            task.skip('No clusters found');
            return subListr;
        }
        subListr.add(clustersArn.map((clusterArn) => {
            const clusterState = {
                arn: clusterArn,
                services: [],
            };
            currentState.push(clusterState);
            return {
                title: clusterArn,
                task: async (ctx, task) => this.getClusterState(task, clusterState),
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 3,
            exitOnError: false,
            collapse: false,
        });
        for (const cluster of currentState) {
            for (const service of cluster.services) {
                subListr.add({
                    title: `${chalk_1.default.blueBright(StopFargateEcsServicesTrick.getEcsServiceResourceId(cluster.arn, service.arn))}`,
                    task: () => new listr_1.default([
                        {
                            title: 'Desired count',
                            task: (ctx, task) => this.conserveDesiredCount(task, cluster, service, options),
                        },
                        {
                            title: 'Auto scaling',
                            task: (ctx, task) => this.conserveScalableTargets(task, cluster, service, options),
                        },
                    ], {
                        exitOnError: false,
                        concurrent: true,
                        collapse: false,
                    }),
                });
            }
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 3,
            exitOnError: false,
            collapse: false,
        });
        for (const cluster of originalState) {
            for (const service of cluster.services) {
                subListr.add({
                    title: `${chalk_1.default.blueBright(StopFargateEcsServicesTrick.getEcsServiceResourceId(cluster.arn, service.arn))}`,
                    task: () => new listr_1.default([
                        {
                            title: 'Desired count',
                            task: (ctx, task) => this.restoreDesiredCount(task, cluster, service, options),
                        },
                        {
                            title: 'Auto scaling',
                            task: (ctx, task) => this.restoreScalableTargets(task, cluster, service, options),
                        },
                    ], {
                        exitOnError: false,
                        concurrent: true,
                        collapse: false,
                    }),
                });
            }
        }
        return subListr;
    }
    async getClusterState(task, clusterState) {
        const services = await this.describeAllServices(task, clusterState.arn);
        if (!services || services.length === 0) {
            task.skip('No services found');
            return;
        }
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        subListr.add(services.map((service) => {
            if (!service.serviceArn) {
                throw new Error(`Unexpected error: Fargate ECS Service does not have ARN`);
            }
            const serviceState = {
                arn: service.serviceArn,
                desired: service.desiredCount,
                scalableTargets: [],
            };
            clusterState.services.push(serviceState);
            return {
                title: service.serviceArn,
                task: async (ctx, task) => this.getServiceState(task, serviceState, clusterState),
            };
        }));
        return subListr;
    }
    async getServiceState(task, serviceState, clusterState) {
        task.output = 'Fetching scalable targets...';
        const scalableTargets = await this.describeAllScalableTargets(clusterState.arn, serviceState.arn);
        serviceState.scalableTargets.push(...scalableTargets.map(st => ({
            namespace: 'ecs',
            resourceId: st.ResourceId,
            scalableDimension: st.ScalableDimension,
            min: st.MinCapacity,
            max: st.MaxCapacity,
        })));
    }
    async listClusters(task) {
        const clustersArn = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        clustersArn.push(...(await this.ecsClient.listClusters({ maxResults: 100 }).promise())
            .clusterArns);
        return clustersArn;
    }
    async listServices(task, clusterArn) {
        const servicesArn = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        servicesArn.push(...(await this.ecsClient
            .listServices({
            cluster: clusterArn,
            launchType: 'FARGATE',
            maxResults: 100,
        })
            .promise()).serviceArns);
        return servicesArn;
    }
    async describeAllServices(task, clusterArn) {
        task.output = `Fetching all services ARNs...`;
        const servicesArn = await this.listServices(task, clusterArn);
        const chunks = lodash_1.default.chunk(servicesArn, 10);
        const result = [];
        for (let i = 0, c = chunks.length; i < c; i++) {
            task.output = `Describing services, page ${i + 1} of ${c}...`;
            const response = await this.ecsClient
                .describeServices({ services: chunks[i], cluster: clusterArn })
                .promise();
            result.push(...(response.services || []));
        }
        return result;
    }
    async describeAllScalableTargets(clusterArn, serviceArn) {
        const result = await this.aasClient
            .describeScalableTargets({
            ServiceNamespace: 'ecs',
            ResourceIds: [
                StopFargateEcsServicesTrick.getEcsServiceResourceId(clusterArn, serviceArn),
            ],
            ScalableDimension: 'ecs:service:DesiredCount',
        })
            .promise();
        return result.ScalableTargets || [];
    }
    async conserveDesiredCount(task, clusterState, serviceState, options) {
        if (serviceState.desired < 1) {
            task.skip(`Skipped, desired count is already zero`);
            return;
        }
        if (options.dryRun) {
            task.skip('Skipped, would set tasks desired count to 0');
            return;
        }
        task.output = `Updating desired count to zero...`;
        await this.ecsClient
            .updateService({
            cluster: clusterState.arn,
            service: serviceState.arn,
            desiredCount: 0,
        })
            .promise();
        task.output = `Waiting for service to scale down to zero...`;
        await this.ecsClient
            .waitFor('servicesStable', {
            cluster: clusterState.arn,
            services: [serviceState.arn],
        })
            .promise();
        task.output = 'Set desired count to zero';
    }
    async conserveScalableTargets(task, clusterState, serviceState, options) {
        if (!serviceState.scalableTargets ||
            serviceState.scalableTargets.length === 0) {
            task.skip('No scalable targets defined');
            return;
        }
        for (const scalableTarget of serviceState.scalableTargets) {
            const resourceId = StopFargateEcsServicesTrick.getEcsServiceResourceId(clusterState.arn, serviceState.arn);
            if (options.dryRun) {
                task.skip('Skipped, would set scalable target min = 0 and max = 0');
            }
            else {
                await this.aasClient
                    .registerScalableTarget({
                    ServiceNamespace: scalableTarget.namespace,
                    ResourceId: resourceId,
                    ScalableDimension: scalableTarget.scalableDimension,
                    MinCapacity: 0,
                    MaxCapacity: 0,
                })
                    .promise();
            }
        }
    }
    async restoreDesiredCount(task, clusterState, serviceState, options) {
        if (serviceState.desired < 1) {
            task.skip(`Skipped, desired count was previously zero`);
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would update desired count to ${serviceState.desired}`);
            return;
        }
        task.output = `Updating desired count to ${serviceState.desired}...`;
        await this.ecsClient
            .updateService({
            cluster: clusterState.arn,
            service: serviceState.arn,
            desiredCount: serviceState.desired,
        })
            .promise();
        task.output = `Waiting for service to reach ${serviceState.desired} desired tasks...`;
        await this.ecsClient
            .waitFor('servicesStable', {
            cluster: clusterState.arn,
            services: [serviceState.arn],
        })
            .promise();
        task.output = `Restored desired count to ${serviceState.desired}`;
    }
    async restoreScalableTargets(task, clusterState, serviceState, options) {
        if (!serviceState.scalableTargets ||
            serviceState.scalableTargets.length === 0) {
            task.skip('No scalable targets defined');
            return;
        }
        for (const scalableTarget of serviceState.scalableTargets) {
            const resourceId = StopFargateEcsServicesTrick.getEcsServiceResourceId(clusterState.arn, serviceState.arn);
            if (options.dryRun) {
                task.skip(`Skipped, would set scalable target min = ${scalableTarget.min} and max = ${scalableTarget.max}`);
            }
            else {
                await this.aasClient
                    .registerScalableTarget({
                    ServiceNamespace: scalableTarget.namespace,
                    ResourceId: resourceId,
                    ScalableDimension: scalableTarget.scalableDimension,
                    MinCapacity: scalableTarget.min,
                    MaxCapacity: scalableTarget.max,
                })
                    .promise();
            }
        }
    }
    static getEcsServiceResourceId(clusterArn, serviceArn) {
        const clusterName = clusterArn
            .toString()
            .split('/')
            .pop();
        const serviceName = serviceArn
            .toString()
            .split('/')
            .pop();
        return `service/${clusterName}/${serviceName}`;
    }
}
exports.StopFargateEcsServicesTrick = StopFargateEcsServicesTrick;
StopFargateEcsServicesTrick.machineName = 'stop-fargate-ecs-services';
//# sourceMappingURL=stop-fargate-ecs-services.trick.js.map