"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StopRdsDatabaseInstancesTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
class StopRdsDatabaseInstancesTrick {
    constructor() {
        this.rdsClient = new aws_sdk_1.default.RDS();
    }
    getMachineName() {
        return StopRdsDatabaseInstancesTrick.machineName;
    }
    getConserveTitle() {
        return 'Stop RDS Database Instances';
    }
    getRestoreTitle() {
        return 'Start RDS Database Instances';
    }
    async getCurrentState(task, currentState, options) {
        const databases = await this.listDatabases(task);
        const subListr = new listr_1.default({
            concurrent: true,
            exitOnError: false,
            collapse: false,
        });
        if (!databases || databases.length === 0) {
            task.skip('No RDS databases found');
            return subListr;
        }
        subListr.add(databases.map((database) => {
            return {
                title: database.DBInstanceIdentifier || chalk_1.default.italic('<no-identifier>'),
                task: async () => {
                    if (database.DBInstanceIdentifier === undefined) {
                        throw new Error(`Unexpected error: DBInstanceIdentifier is missing for RDS database`);
                    }
                    if (database.DBInstanceStatus === undefined) {
                        throw new Error(`Unexpected error: DBInstanceStatus is missing for RDS database`);
                    }
                    currentState.push({
                        identifier: database.DBInstanceIdentifier,
                        status: database.DBInstanceStatus,
                    });
                },
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const database of currentState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${database.identifier}`),
                    task: (ctx, task) => this.conserveDatabase(task, database, options),
                });
            }
        }
        else {
            task.skip(`No RDS databases found`);
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (originalState && originalState.length > 0) {
            for (const database of originalState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${database.identifier}`),
                    task: (ctx, task) => this.restoreDatabase(task, database, options),
                });
            }
        }
        else {
            task.skip(`No RDS databases was conserved`);
        }
        return subListr;
    }
    async conserveDatabase(task, databaseState, options) {
        if (databaseState.status !== 'available') {
            task.skip(`Skipped, current state is not "available" it is "${databaseState.status}" instead`);
            return;
        }
        if (options.dryRun) {
            task.skip('Skipped, would stop the RDS instance');
            return;
        }
        await this.rdsClient
            .stopDBInstance({
            DBInstanceIdentifier: databaseState.identifier,
        })
            .promise();
        // TODO Find a way to wait for "stopped" state
        task.output = 'Stopped successfully';
    }
    async restoreDatabase(task, databaseState, options) {
        if (databaseState.status !== 'available') {
            task.skip(`Skipped, previous state was not "available" it was "${databaseState.status}" instead`);
            return;
        }
        if (options.dryRun) {
            task.skip('Skipped, would start RDS instance');
            return;
        }
        try {
            task.output = 'Starting RDS instance...';
            await this.rdsClient
                .startDBInstance({
                DBInstanceIdentifier: databaseState.identifier,
            })
                .promise();
            task.output = 'Waiting for RDS instance to be available...';
            await this.rdsClient
                .waitFor('dBInstanceAvailable', {
                DBInstanceIdentifier: databaseState.identifier,
            })
                .promise();
            task.output = 'Started successfully';
        }
        catch (error) {
            if (error.code === 'InvalidDBInstanceState') {
                task.skip('Skipped, database is not in "stopped" state.');
                return;
            }
            throw error;
        }
    }
    async listDatabases(task) {
        const databases = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        databases.push(...((await this.rdsClient
            .describeDBInstances({
            MaxRecords: 100,
        })
            .promise()).DBInstances || []).filter(instance => !instance.DBClusterIdentifier));
        return databases;
    }
}
exports.StopRdsDatabaseInstancesTrick = StopRdsDatabaseInstancesTrick;
StopRdsDatabaseInstancesTrick.machineName = 'stop-rds-database-instances';
//# sourceMappingURL=stop-rds-database-instances.trick.js.map